#!/bin/bash

function install_daqgroup_markdown_viewer(){
 function show_greetings() {
  echo " ";
  echo " Install: DaqGroup Markdown Viewer addon for Firefox …";
  echo " Copyright (c) 2023 Alexey Kuryakin kouriakine@mail.ru";
  echo " Version: 20230411";
  echo "";
 };
 function note(){ return; };
 function fatal(){ echo -ne "\n$1\n\n"; exit 1; };
 function show_title() {
  echo " ";
  echo " $*";
  echo " ";
 };
 function execute(){
  echo "";
  echo "$(date +%Y.%m.%d-%H:%M:%S) - Execute: $*";
  echo "";
  $*;
 };
 function print_mime_types(){
  echo "text/html                   html htm";
  echo "text/plain                  txt text";
  echo "text/plain                  md mkd markdown";
  echo "text/plain                  lm9 Lm9 lM9 LM9";
  echo "text/plain                  pas inc dpr lpr";
  echo "text/plain                  cfg crc sml log";
  echo "text/plain                  bat cmd vbs lua wlua";
  echo "application/x-shellscript   sh run";
  echo "application/x-deb           deb";
 };
 note clear;
 show_greetings;
 note "######################";
 note "Script identification.";
 note "######################";
 local startupdir="$(pwd -LP)";
 local scriptfile="${BASH_SOURCE[0]}";
 local scriptname="$(basename $scriptfile)";
 local scriptbase="$(basename $scriptfile .sh)";
 local scripthome="$(readlink -f $(dirname $scriptfile))";
 local scriptFILE="$(readlink -f $scriptfile)";
 local scriptHOME="$(readlink -f $(dirname $scriptFILE))";
 local date="$(date +%Y.%m.%d-%H:%M:%S)";
 note "##################################";
 show_title "Set mime types for Firefox …";
 note "##################################";
 local file_mime_types="$HOME/.mime.types";
 print_mime_types > $file_mime_types;
 execute cat        $file_mime_types;
 note "##################################";
 show_title "Copy markdown-mark pixmaps …";
 note "##################################";
 local pixmaps="/usr/share/pixmaps";
 local imglist="markdown-mark.png markdown-mark.svg markdown-mark.xpm";
 for img in $imglist; do
  if [ -e "$scripthome/$img" ]; then
   if [ -e "$pixmaps/$img" ]; then
    echo "Already exist: $pixmaps/$img";
   else
    sudo cp -fv $scripthome/$img $pixmaps/$img;
   fi;
  fi;
 done;
 if which xdg-icon-resource; then
  for usr in root $USER; do
   local run=""; local mod="user";
   if [ "$usr" = "root" ]; then run="sudo"; mod="system"; fi;
   for img in $imglist; do
    if [ -e "$scripthome/$img" ]; then
     case $img in
      *.png|*.xpm) $run xdg-icon-resource install --noupdate --novendor --mode $mod --size 48 $scripthome/$img; ;;
      *) ;;
     esac;
    fi;
   done;
   $run xdg-icon-resource forceupdate;
  done;
 fi;
 note "#########################################";
 show_title "Update mime/applications database …";
 note "#########################################";
 local sharetar="$scripthome/markdownshare.tar";
 if [ -e "$sharetar" ]; then
  local opt="--no-same-owner";
  if tar -xvf "$sharetar" -C $HOME/.local/share/ $opt; then
   if update-mime-database $HOME/.local/share/mime; then echo "MIME database updated for $USER"; fi;
   if update-desktop-database $HOME/.local/share/applications; then echo "Desktop database updated for $USER."; fi;
  fi;
 fi;
 local defapp="";
 local appnum="2";
 case $appnum in
  1) defapp="markdownviewer-open.desktop"; ;;
  2) defapp="markdownviewer-browse.desktop"; ;;
  3) defapp="markdownviewer-convert.desktop"; ;;
  4) defapp="markdownviewer-open-chromium.desktop"; ;;
  5) defapp="markdownviewer-browse-chromium.desktop"; ;;
  *) defapp="firefox.desktop"; ;;
 esac;
 case $(xdg-mime query default text/markdown) in
  markdownviewer-browse.desktop|markdownviewer-open.desktop) note "Already valid."; ;;
  *) if [ -n "$defapp" ]; then gio mime text/markdown $defapp; fi; ;;
 esac;
 echo ""
 echo " MIME information on text/markdown:";
 echo "";
 gio mime text/markdown;
 note "##########################################";
 show_title "Create symlinks for required files …";
 note "##########################################";
 local targetdir="/opt/daqgroup/share/daqgroup-markdown-viewer";
 if [ ! -e $targetdir ]; then targetdir="$scripthome"; fi;
 sudo rm -fv /usr/local/bin/markdownviewer;
 sudo rm -fv /usr/local/bin/markdownviewer.sh;
 sudo ln -s -T "$targetdir/markdownviewer.sh" /usr/local/bin/markdownviewer;
 sudo ln -s -T "$targetdir/markdownviewer.sh" /usr/local/bin/markdownviewer.sh;
 sudo chmod -c 755 "$targetdir/markdownviewer.sh";
 note "########################################";
 show_title "Try to install Firefox addon XPI …";
 note "########################################";
 local xpi="$(find $scripthome/daqgroup_markdown_viewer-*.xpi -type f | sort | tail -n 1)";
 if [ -n "$xpi" ] && [ -e "$xpi" ]; then
  echo "Found XPI: $(basename $xpi)";
  note "Execute Firefox detached …";
  show_title firefox file://$xpi;
             firefox file://$xpi >/dev/null 2>&1 & disown;
  if which 7za >/dev/null 2>&1; then
   local dir="$targetdir/$(basename $xpi .xpi)";
   echo "Unpack $xpi archive to directory $dir …";
   if [ -e "$dir/" ]; then sudo rm -fvr $dir; fi;
   sudo mkdir -p $dir; sudo chmod 755 $dir;
   sudo 7za x -bb1 -tzip $xpi -o$dir;
   sudo chmod -c ugo+xr $(sudo find $dir/* -type d);
   sudo chmod -c ugo+r  $(sudo find $dir/* -type f);
  fi;
 else
  local bug="Error: could not find *.xpi file.";
  notify-send -t 15000 -i dialog-error "$date: $scriptname" "$bug";
  echo -ne "\n\n$bug\n\n";
 fi;
 show_title "Done $scriptname";
};
install_daqgroup_markdown_viewer;
