#!/bin/bash

function markdownviewer_sh(){
 function about(){
  echo "$(basename ${BASH_SOURCE[0]}) version 20230324";
  echo "##############################################################";
  echo "# MarkdownViewer: convert .md to .md.html and browse HTML.   #";
  echo "# Copyright (c) 2023 Alexey Kuryakin kouriakine@mail.ru      #";
  echo "# Under MIT License, see https://opensource.org/licenses/MIT #";
  echo "##############################################################";
  echo "# The script to convert and view Markdown files via Browser. #";
  echo "# Uses (pandoc) or (perl markdown.pl) to convert MD to HTML. #";
  echo "##############################################################";
  echo "# Usage: markdownviewer file.md [--options [browser]]        #";
  echo "# Options: --browse|--open|--convert-only                    #";
  echo "# Browser: firefox|chromium                                  #";
  echo "##############################################################";
 };
 function note(){ return; };
 note "####################";
 note "Fatal error handler.";
 note "####################";
 function fatal(){
  local date="$(date +%Y.%m.%d-%H:%M:%S)"; local script="$(basename ${BASH_SOURCE[0]})";
  if which notify-send >/dev/null 2>&1; then notify-send -t 30000 -i dialog-error "$date: $script" "$1"; fi;
  echo -ne "\n$1\n\n";
  exit 1;
 };
 note "#######################";
 note "Print user/script info.";
 note "#######################";
 function print_identification(){
  echo "UserID: $USER@$HOSTNAME on $(lsb_release -ds)";
  echo "RunDir: $startupdir";
  echo "Folder: $scriptHOME";
  echo "Locate: $scripthome";
  echo "Script: $scriptname";
 };
 note "############################";
 note "Check packages is installed.";
 note "############################";
 function check_install_package(){
  while [ -n "$1" ]; do
   local program="$(echo "$1" | cut -d : -f 1)";
   local package="$(echo "$1" | cut -d : -f 2)";
   echo -n "Check $program from package $package: "
   if [ -z "$(which $program)" ]; then sudo apt install $1; fi;
   if [ -n "$(which $program)" ]; then echo "OK"; else echo "FAIL"; fatal "Error: missed package $package."; fi;
   shift;
  done;
 };
 note "###########################";
 note "First argument is a nimber?";
 note "###########################";
 function is_number(){
  case $1 in
   ''|*[!0-9]*)  return 1; ;;
   *)            return 0; ;;
  esac;
 };
 note "##################";
 note "Is I am root user?";
 note "##################";
 function is_iam_root(){
  if [ "$(whoami)" = "root" ]; then return 0; else return 1; fi;
 };
 note "############################";
 note "Argument is valid user name?";
 note "is_valid_user foo && echo Ok";
 note "############################";
 function is_valid_user(){
  if [ -n "$1" ]; then
   if id $1 >/dev/null 2>&1; then
    return 0;
   fi;
  fi;
  return 1;
 };
 note "#################################";
 note "Print current X DISPLAY user name";
 note "#################################";
 function print_x_display_user(){
  local xuser="$(who | grep -e "\s$DISPLAY\s" | head -n 1 | awk '{print $1}')";
  if is_valid_user "$xuser"; then echo "$xuser"; return 0; else return 1; fi;
 };
 note "#######################";
 note "Check file is Markdown.";
 note "#######################";
 function CheckArguments(){
  if [ -z "$1" ];   then fatal "Error: markdown file name expected."; fi;
  if [ ! -e "$1" ]; then fatal "Error: markdown file was not found."; fi;
 };
 function DetectMarkdown(){
  case $1 in
   *.md|*.mkd|*.mkdn|*.mdwn|*.mdown|*.markdown) note "text/markdown detected."; ;;
   *) fatal "Error: file $1 is not recognized as Markdown."; ;;
  esac;
  echo "Try open text/markdown: $1";
 };
 note "##########################";
 note "Set pandoc dialect/options";
 note "##########################";
 function PandocOptions(){
  note "highlight styles: pygments kate monochrome espresso zenburn haddock tango breezedark";
  pandopt="--highlight-style=pygments";
 };
 function PandocDialect(){
  PandocOptions;
  dialect="commonmark";
  dialect="markdown_mmd";
  dialect="markdown_strict";
  dialect="markdown_github";
  dialect="markdown_phpextra";
  dialect="markdown";
  note "Add next Extensions:";
  dialect="$dialect+emoji";
  dialect="$dialect+abbreviations";
 };
 note "#####################################################";
 note "Print CSS styles for HTML; located in scriptname.css.";
 note "#####################################################";
 function HtmlCssDefaults(){
  echo "body { background-color: #F7F7F7; }";
  echo "body, code, pre, tt { font-family:'PT Mono','Liberation Mono','Andale Mono','Courier New',monospace; }";
  echo "body { font-size:12pt; } code, pre, tt { font-size:12pt; }";
  echo "pre, tt { border: 1px solid red; border-radius: 3px; padding: 4px 8px; }";
  echo "code, pre, tt { background-color: mistyrose; }";
  echo "table, tr, th, td { border: 1px solid gray; }";
  echo "td { background-color: #F0F0F0; }";
  echo "th { background-color: silver; }";
  echo "td, th { padding: 4px 8px; }";
  echo "blockquote { border: 1px dotted silver; border-left: 8px solid silver; padding: 0 15px; color: navy; background-color: lightyellow; }";
 };
 function HtmlCssStyles(){
  local markdowncss="$scriptHOME/$scriptbase.css";
  if [ -e "$markdowncss" ]; then cat $markdowncss; else HtmlCssDefaults; fi;
 };
 note "#####################################################";
 note "Print META tags for HTML (located in scriptname.meta)";
 note "#####################################################";
 function HtmlMetaDefaults(){
  echo "<meta http-equiv=\"Content-Language\" Content=\"ru\">";
  echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">";
 };
 function HtmlMetaTags(){
  local markdownmeta="$scriptHOME/$scriptbase.meta";
  if [ -e "$markdownmeta" ]; then cat $markdownmeta; else HtmlMetaDefaults; fi;
 };
 note "################################################";
 note "Print HTML head and tail to compose result HTML.";
 note "HTML body produced with pandoc|markdown command.";
 note "################################################";
 function HtmlInfo(){
  echo " /*";
  echo " Generated $date with command: $*";
  echo " */";
 };
 function HtmlHead(){
  echo "<html>";
  echo "<head>";
  HtmlMetaTags;
  echo "<style>";
  HtmlInfo "$@";
  HtmlCssStyles;
  echo "</style>";
  echo "</head>";
  echo "<body>";
 };
 function HtmlTail(){
  echo "</body>";
  echo "</html>";
 };
 note "####################################";
 note "Detect [[toc]] line is exist in file";
 note "[[toc]] must be single token in line";
 note "####################################";
 function DetectTocLine(){
  if [ -n "$1" ] && [ -e "$1" ]; then
   if grep -Pi '^\s*\[\[toc\]\]\s*$' "$1" >/dev/null 2>&1; then
    echo "TOC expression found in $1.";
    return 0;
   fi;
  fi;
  return 1;
 };
 note "###################################";
 note "Set variable as number (calculate).";
 note "Example: SetVarNumber size '1024*8'";
 note "###################################";
 function SetVarNumber(){
  if [ -n "$1" ] && [ -n "$2" ]; then
   let $1="$2";
  fi;
 };
 note "######################################";
 note "Get line number of [[toc]] to variable";
 note "Example: GetTocLineNum TocLine abc.md ";
 note "######################################";
 function GetTocLineNum(){
  if [ -n "$1" ]; then
   SetVarNumber $1 0;
   if [ -n "$2" ] && [ -e "$2" ]; then
    local num="$(cat "$2"| grep -Pin "^(\x3Cp\x3E)*\s*\[\[toc\]\]\s*(\x3C/p\x3E)*\s*$" | cut -d : -f 1 | head -n 1)";
    if is_number "$num"; then SetVarNumber $1 "$num"; fi;
   fi;
  fi;
 };
 note "######################################";
 note "Handle [[toc]] expression if one found";
 note "Usage: HandleTocLine file.md temp.html";
 note "######################################";
 function HandleTocLine(){
  echo 'Handle TOC expression.';
  if [ -n "$1" ] && [ -e "$1" ] && [ -n "$2" ] ; then
   GetTocLineNum TocLineMd "$1";
   if [ $TocLineMd -le 0 ]; then
    echo "TOC line not found.";
    return 0;
   else
    echo "TOC line MD $TocLineMd";
   fi;
   local TargetHtml="$2";
   local TempFile1="$tempdir/$(basename $1).tmp.1.html";
   local TempFile2="$tempdir/$(basename $1).tmp.2.html"
   GetTocLineNum TocLineHtml "$TargetHtml";
   SetVarNumber  TocLineHead "$TocLineHtml-1";
   echo "TOC line $TocLineHtml, head $TocLineHead";
   if [ $TocLineHtml -gt 0 ] && [ $TocLineHead -gt 0 ]; then
    cat "$1" | pandoc -f $dialect -t html $pandopt --standalone --toc > "$TempFile1";
    SetVarNumber TocLineBeg 0;
    local beg="$(cat "$TempFile1" | grep -Pin '^\s*\x3Cdiv id\=\x22TOC\x22\x3E\s*$' | cut -d : -f 1 | head -n 1)";
    if is_number "$beg"; then SetVarNumber TocLineBeg "$beg"; fi;
    if [ $TocLineBeg -gt 0 ]; then
     SetVarNumber TocLineEnd 0;
     local end="$(cat "$TempFile1" | grep -Pin '^\s*\x3C/div\x3E\s*$' | cut -d : -f 1 | head -n 1)";
     if is_number "$end"; then SetVarNumber TocLineEnd $end; fi;
     SetVarNumber TocLineCount "$TocLineEnd-$TocLineBeg+1";
     if [ $TocLineEnd -gt 0 ] && [ $TocLineCount -gt 0 ]; then
      echo "TOC div block $TocLineEnd,$TocLineBeg,$TocLineCount";
      cat "$TargetHtml" 2>/dev/null | head -n $TocLineHead                         >  "$TempFile2"
      cat "$TempFile1"  2>/dev/null | head -n $TocLineEnd | tail -n $TocLineCount  >> "$TempFile2"
      cat "$TargetHtml" 2>/dev/null | sed "1,${TocLineHtml}d"                      >> "$TempFile2"
      cp -fv "$TempFile2" "$TargetHtml";
     fi;
    fi;
   fi;
   rm -fv "$TempFile1" "$TempFile2";
  fi;
  echo "Done handle TOC expression.";
 };
 note "####################################";
 note "Try to start Browser for given file.";
 note "Skip if opt_convert_only flag is ON.";
 note "####################################";
 function StartBrowser(){
  if [ "$opt_convert_only" = "1" ]; then return 0; fi;
  if [ -z "$1" ];   then fatal "Error: browse file undefined."; fi;
  if [ ! -e "$1" ]; then fatal "Error: browse file is missed."; fi;
  local run=""; note "The prefix to call Browser as user who called sudo.";
  if is_iam_root; then
   note "Browser disabled for root.";
   note "Need sudo to call as user.";
   note "Try detect X session user:";
   local xuser="$(print_x_display_user)";
   for iter in 1; do
    if is_valid_user "$xuser" && [ "$xuser" != "root" ]; then echo "xuser[who]: $xuser"; break; else xuser="$(basename $(dirname $XAUTHORITY))"; fi;
    if is_valid_user "$xuser" && [ "$xuser" != "root" ]; then echo "xuser[xau]: $xuser"; break; else xuser="$SUDO_USER"; fi;
    if is_valid_user "$xuser" && [ "$xuser" != "root" ]; then echo "xuser[sud]: $xuser"; break; else echo "xuser fail."; fi;
   done;
   if is_valid_user "$xuser" && [ "$xuser" != "root" ]; then run="sudo -u $xuser -H -i -n"; fi;
  fi;
  local Browser="$(echo $browser | awk '{print $1}')";
  if which $Browser >/dev/null 2>&1; then
   echo "$run $browser $1";
         $run $browser $1 >/dev/null 2>&1 & disown;
   return $?;
  else
   fatal "Error: $Browser is not found.";
  fi;
 };
 note "#####################################";
 note "Check and copy file source -> target.";
 note "Usage: CheckCopyFile source target   ";
 note "#####################################";
 function CheckCopyFile(){
  if [ -n "$1" ] && [ -e "$1" ] && [ -n "$2" ]; then
   if diff "$1" "$2" >/dev/null 2>&1; then
    echo "No need to update $2.";
    return 0;
   else
    if cp -fv "$1" "$2"; then
     return 0;
    else
     echo "Could not copy $1 to $2.";
    fi;
   fi;
  fi;
  return 1;
 };
 note "#####################################################";
 note "Convert .md to .md.html with pandoc or markdown tool.";
 note "Usage: ConvertWithPandoc src.md temp.html target.html";
 note "#####################################################";
 function ConvertWithPandoc(){
  if [ -n "$1" ] && [ -n "$2" ] && [ -n "$3" ]; then
   echo "Try convert with pandoc: $1 -> $2";
   PandocDialect; note "Assign pandoc dialect and options.";
   HtmlHead   pandoc -f $dialect -t html $pandopt   >  "$2";
   cat "$1" | pandoc -f $dialect -t html $pandopt   >> "$2";
   HtmlTail                                         >> "$2";
   echo "Done convert with pandoc.";
   if DetectTocLine "$1"; then HandleTocLine "$1" "$2"; fi;
   local html="$2";
   if CheckCopyFile "$2" "$3"; then html="$3"; fi;
   StartBrowser "$html";
   return $?;
  else
   fatal "Error: invalid params ($1,$2,$3).";
  fi;
 };
 function ConvertWithMarkdown(){
  if [ -n "$1" ] && [ -n "$2" ] && [ -n "$3" ]; then
   echo "Try convert with markdown: $1 -> $2";
   HtmlHead   markdown --html4tags    >  "$2";
   cat "$1" | markdown --html4tags    >> "$2";
   HtmlTail                           >> "$2";
   echo "Done convert with markdown.";
   local html="$2";
   if CheckCopyFile "$2" "$3"; then html="$3"; fi;
   StartBrowser "$html";
   return $?;
  else
   fatal "Error: invalid params ($1,$2,$3).";
  fi;
 };
 function print_goodbye(){
  echo "";
  echo "Done $scriptname.";
  echo "";
 };
 note "######################";
 note "Script identification.";
 note "######################";
 local startupdir="$(pwd -LP)";
 local scriptfile="${BASH_SOURCE[0]}";
 local scriptname="$(basename $scriptfile)";
 local scriptbase="$(basename $scriptfile .sh)";
 local scripthome="$(readlink -f $(dirname $scriptfile))";
 local scriptFILE="$(readlink -f $scriptfile)";
 local scriptHOME="$(readlink -f $(dirname $scriptFILE))";
 local date="$(date +%Y.%m.%d-%H:%M:%S)";
 note "######################";
 note "Local script variables";
 note "######################";
 local tempdir="/tmp/daqgroup-$USER/$scriptbase";
 local browser="firefox --new-tab";
 local opt_convert_only=0;
 local opt_browse=0;
 local pandopt="";
 local dialect="";
 note "########";
 note "# Main #";
 note "########";
 function main(){
  about; print_identification;
  check_install_package markdown:markdown pandoc:pandoc firefox:firefox-astra chromium:chromium;
  CheckArguments "$@";
  DetectMarkdown "$1";
  local md="$(readlink -f $1)";
  echo "Source: $md";
  note "####################";
  note "Handle args,options.";
  note "####################";
  if [ -n "$2" ]; then
   case $2 in
    --convert-only)  opt_convert_only=1; ;;
    --browse)        opt_browse=1; ;;
    --open)          note "Default action."; ;;
    '')              note "No options set."; ;;
    *)               fatal "Error: unknown option: $2."; ;;
   esac;
  fi;
  if [ -n "$3" ]; then
   local bro="$(echo $3 | cut -d ' ' -f 1)";
   case $bro in
    firefox|chromium) if which $bro >/dev/null 2>&1; then browser="$3"; fi; ;;
    *) fatal "Error: expected supported browser (firefox|chromium)."; ;;
   esac;
  fi;
  if [ $opt_browse -eq 1 ]; then
   StartBrowser $md;
   print_goodbye;
   return $?;
  fi;
  note "####################";
  note "Make temp directory.";
  note "####################";
  if [ ! -e $tempdir/ ]; then
   if mkdir -p $tempdir; then
    echo "mkdir $tempdir";
   else
    fatal "Error: could not mkdir $tempdir.";
   fi;
  fi;
  note "############################################";
  note "Convert .md to .md.html; use temp html file.";
  note "############################################";
  local targhtml="$md.html"; note "target html";
  local temphtml="$tempdir/$(basename $md).html";
  for iter in 1; do
   if which pandoc >/dev/null 2>&1; then
    rm -fv "$temphtml"; note "remove old temp html";
    ConvertWithPandoc "$md" "$temphtml" "$targhtml";
    break;
   fi;
   if which markdown >/dev/null 2>&1; then
    rm -fv "$temphtml"; note "remove old temp html";
    ConvertWithMarkdown "$md" "$temphtml" "$targhtml";
    break;
   fi;
  done;
  print_goodbye;
 };
 main "$@";
};
markdownviewer_sh "$@";

#############
# END OF FILE
#############
